<?php

namespace Bmc\Staff\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Bmc\Staff\Forms\Staff as StaffForm;
use Bmc\Staff\Http\Requests\StaffRequest;
use Bmc\Staff\Interfaces\StaffRepositoryInterface;
use Bmc\Staff\Repositories\Eloquent\Filters\StaffResourceFilter;
use Bmc\Staff\Repositories\Eloquent\Presenters\StaffListPresenter;

/**
 * Resource controller class for staff.
 */
class StaffResourceController extends BaseController
{

    /**
     * Initialize staff resource controller.
     *
     *
     * @return null
     */
    public function __construct(StaffRepositoryInterface $staff)
    {
        parent::__construct();
        $this->form = StaffForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('bmc.staff.modules'), 'staff', guard_url('staff'));
        $this->repository = $staff;
    }

    /**
     * Display a list of staff.
     *
     * @return Response
     */
    public function index(StaffRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(StaffResourceFilter::class)
            ->setPresenter(StaffListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('staff::staff.names'))
            ->view('staff::staff.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display staff.
     *
     * @param Request $request
     * @param Model   $staff
     *
     * @return Response
     */
    public function show(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('staff::staff.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('staff::staff.show')
            ->output();
    }

    /**
     * Show the form for creating a new staff.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('staff::staff.name'))
            ->view('staff::staff.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new staff.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('staff::staff.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('staff/staff/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/staff/staff'))
                ->redirect();
        }

    }

    /**
     * Show staff for editing.
     *
     * @param Request $request
     * @param Model   $staff
     *
     * @return Response
     */
    public function edit(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('staff::staff.name'))
            ->view('staff::staff.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the staff.
     *
     * @param Request $request
     * @param Model   $staff
     *
     * @return Response
     */
    public function update(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('staff::staff.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('staff/staff/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('staff/staff/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the staff.
     *
     * @param Model   $staff
     *
     * @return Response
     */
    public function destroy(StaffRequest $request, StaffRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('staff::staff.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('staff/staff/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('staff/staff/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
